#import numpy

import GEOM
import geompy
import math
import SALOMEDS
import salome
import SMESH
import smesh
import NETGENPlugin
import StdMeshers


def to_string(in_int):
    """Converts an integer to a string"""
    out_str = ""
    prefix = ""
    if in_int < 0:
        prefix = "-"
        in_int = -in_int
    while in_int / 10 != 0:
        out_str = chr(ord('0') + in_int % 10) + out_str
        in_int = in_int / 10
    out_str = chr(ord('0') + in_int % 10) + out_str
    return prefix + out_str


x0 = 0
x1 = 2
y1 = 2
z1 = 3

pcyl = geompy.MakeVertex(x0, y1, z1)
p2 = geompy.MakeVertex(x1, y1, z1)
Vcyl = geompy.MakeLineTwoPnt(pcyl, p2)
block = geompy.MakeBoxDXDYDZ(2, 3.5, 20)

geompy.addToStudy( pcyl, "pcyl" )
geompy.addToStudy( p2, "p2" )
geompy.addToStudy( Vcyl, "Vcyl" )
geompy.addToStudy( block, "block" )

Rcylinder = 0.45
Lcylinder = 2.00
clind = geompy.MakeCylinder(pcyl, Vcyl, Rcylinder, Lcylinder)
blockcyl = geompy.MakeCut(block, clind)

geompy.addToStudy( clind, "clind" )
geompy.addToStudy( block, "block" )
geompy.addToStudy( Vcyl, "Vcyl" )
geompy.addToStudy( blockcyl, "blockcyl" )

Pbot = geompy.CreateGroup(blockcyl, geompy.ShapeType["FACE"])
geompy.UnionIDs(Pbot, [13])
Ptop = geompy.CreateGroup(blockcyl, geompy.ShapeType["FACE"])
geompy.UnionIDs(Ptop, [28])
Pcyl = geompy.CreateGroup(blockcyl, geompy.ShapeType["FACE"])
geompy.UnionIDs(Pcyl, [41])

Ltop = geompy.CreateGroup(blockcyl, geompy.ShapeType["EDGE"])
geompy.UnionIDs(Ltop, [32])
Lcyl = geompy.CreateGroup(blockcyl, geompy.ShapeType["EDGE"])
geompy.UnionIDs(Lcyl, [35])



geompy.UnionIDs(Lcyl, [3])
geompy.addToStudyInFather( blockcyl, Pbot, "Pbot" )
geompy.addToStudyInFather( blockcyl, Pcyl, "Pcyl" )
geompy.addToStudyInFather( blockcyl, Ptop, "Ptop" )
geompy.addToStudyInFather( blockcyl, Ltop, "Ltop" )
geompy.addToStudyInFather( blockcyl, Lcyl, "Lcyl" )


## mesh part
## mesh blockcyl
Mbcyl = smesh.Mesh(blockcyl)
Netgen_1D_2D_3D = Mbcyl.Tetrahedron(algo=smesh.FULL_NETGEN)
NETGEN_3D_Parameters = Netgen_1D_2D_3D.Parameters()
NETGEN_3D_Parameters.SetMaxSize( 2 )
NETGEN_3D_Parameters.SetSecondOrder( 1 )
NETGEN_3D_Parameters.SetOptimize( 1 )
NETGEN_3D_Parameters.SetFineness( 2 )
isDone = Mbcyl.Compute()

Pbot = Mbcyl.Group(Pbot)
Pcyl = Mbcyl.Group(Pcyl)
Ptop = Mbcyl.Group(Ptop)
Ltop = Mbcyl.Group(Ltop)
Lcyl = Mbcyl.Group(Lcyl)

smesh.SetName(Mbcyl.GetMesh(), 'Mbcyl')
smesh.SetName(Netgen_1D_2D_3D.GetAlgorithm(), 'Netgen 1D-2D-3D')
smesh.SetName(NETGEN_3D_Parameters, 'NETGEN 3D Parameters')

smesh.SetName(Pbot, 'Pbot')
smesh.SetName(Pcyl, 'Pcyl')
smesh.SetName(Ptop, 'Ptop')
smesh.SetName(Ltop, 'Ltop')
smesh.SetName(Lcyl, 'Lcyl')

#Ltop_2 = Mbcyl.Group(Ltop)
#Ltop_3 = Mbcyl.GroupOnGeom(Ltop,'Ltop',SMESH.NODE)


aFilterManager = smesh.CreateFilterManager()
aFilterA = aFilterManager.CreateFilter()
aCriteria = []
aCriterion = SMESH.Filter.Criterion(18,32,0,'Pcyl','',32,32,0.001,SMESH.NODE,-1)
aBelongToCylinderA = aFilterManager.CreateBelongToCylinder()
aBelongToCylinderA.SetTolerance(0.001)
aCriteria.append(aCriterion)                          

aFilterA.SetCriteria(aCriteria)                       
NodesOnCylinder = aFilterA.GetElementsId(Mbcyl.mesh)              
aFilterA.SetPredicate(aBelongToCylinderA)             
Ncyl = Mbcyl.CreateEmptyGroup( SMESH.NODE, 'Ncyl' )   
nbAdd = Ncyl.Add(NodesOnCylinder)                               

print nbAdd
print NodesOnCylinder

###Salome: http://www.salome-platform.org/forum/forum_11/332108298

node_oncyl = []
zoffset=0
countcyl=0
for allnode in NodesOnCylinder:
    q1, q2, q3 = Mbcyl.GetNodeXYZ(allnode)
    countcyl+=1
    nodestr = to_string(allnode)
    node_oncyl.append(allnode)
    ss = "\n          _F(NOEUD=('N"+nodestr+"','N"+nodestr+"'),DDL=('DY','DZ'),COEF_MULT=("
    print ss,
    print (q2-y1),
    print ",",
    print (q3-z1-zoffset),
    print "),COEF_IMPO=0.00),",
pass
print "),);"
print "\nend of copy and past",
print "\n********************\n\n\n"

print "=Number of nodes on cylinder: ",countcyl
print node_oncyl   # should be the same as NodesOnCylinder ;-)
print NodesOnCylinder



if salome.sg.hasDesktop():
		salome.sg.updateObjBrowser(0)


